import tempfile

from unittest import TestCase
from Freya_alerce.core.commands.base_freya import Base

class TestCreateModule(TestCase):
    
    def setUp(self):
        self.tmp_test = tempfile.TemporaryDirectory()

    def test(self):
        Base(name='test_api',source='api',path=self.tmp_test.name).create_module_catalog()
        Base(name='test_db',source='db',path=self.tmp_test.name).create_module_catalog()
        #Base(name='test_db',source='other',path=self.tmp_test.name).create_module_catalog()

    def tearDown(self):
        self.tmp_test.cleanup()

if __name__ == '__main__':
    unittest.main() 


import os
import sys
import subprocess
import tempfile
import shutil
import zipfile #read zip files
import Freya_alerce.files # __path__
from git import Repo
from Freya_alerce.files.verify_file import Verify
from Freya_alerce.files.list_file import ListFiles

class BaseAPI(object):
    """
    Base class to command line Freya for new api (FreyaAPI) and add resources in FreyaAPI.

    Parameters
    --------------------------------------
    name : (string) 
        name catalog inside Freya with add resource in FreyaAPI
    path : (string) 
        path where created FreyaAPI, local folder for catalogs and
        add resources in FreyaAPI.
    """

    def __init__(self,**kwargs):
        self.name = kwargs.get('name')
        if self.name:
            #self.name = self.name.replace(self.name[0],self.name[0].upper(),1)
            self.name = self.name.upper()
        self.path = kwargs.get('path')    
    
    
    def create_new_api(self):
        """
        Create a new FreyaAPI, the new api created in path
        with call the freya-admin --newapi
        """
        # Path create new api
        path_new_api =  os.path.join(self.path,'FreyaAPI')
        try: 
            tmpdir = tempfile.TemporaryDirectory() # path temp
            Repo.clone_from("https://github.com/fernandezeric/FreyaAPI", tmpdir.name)
            path_temp = os.path.join(tmpdir.name,'FreyaAPI')
            subprocess.check_call([sys.executable, '-m','pip', 'install','-r',os.path.join(path_temp,'requirements.txt')])
            shutil.copytree(path_temp,path_new_api)
            tmpdir.cleanup()
        except OSError as error:
            print(error)    
    
    def create_new_resource(self):
        """
        Add resource to FreyaAPI, first verify the catalog exist inside Freya 
        or in the local catalogs folder.
        """
        # Verify 
        if not Verify().verify_catalog_inside(self.name) and not Verify().verify_catalog_local(self.name):
            raise TypeError ('First created catalog inside Freya or local ')
        
        # Get path to template files
        path_template_resource = ListFiles().path_files_resource()    # Path FreyaAPI 
        path_api = self.path
        if path_api.split('/')[-1] != 'FreyaAPI':
            raise TypeError ('Needs to be on the root path of FreyaAPI')
        else:
            path_new_resource = os.path.join(path_api,f'app/main/resources_freya/{self.name}_resource')

        try: 
            with tempfile.TemporaryDirectory() as tmpdir:
                #print('created temporary directory', tmpdirname)
                extract_zip = zipfile.ZipFile(path_template_resource)
                extract_zip.extractall(tmpdir)
                extract_zip.close()
                list_path = [os.path.join(tmpdir,'resource.py')]
                ListFiles().replace_in_files(list_path,'NAME',self.name)
                shutil.copytree(tmpdir,path_new_resource)
        except OSError as error:
            print(error)    


from unittest import TestCase, mock
from Freya_alerce.catalogs.core.data_lc import DataLcDegree,DataLcHms

class TestMethodsGetData(TestCase):
    
    def setUp(self):
        self.test_get_data_degree_all = DataLcDegree(catalog='test1',ra=139.33444972,dec=68.6350604,nearest=True)
        self.test_get_data_degree_one = DataLcDegree(catalog='test1',ra=139.33444972,dec=68.6350604,nearest=False)
        self.test_get_data_hms_all = DataLcHms(catalog='test2',hms = '9h17m20.26793280000689s +4h34m32.414496000003936s',nearest=True)
        self.test_get_data_hms_one = DataLcHms(catalog='test2',hms = '9h17m20.26793280000689s +4h34m32.414496000003936s',nearest=False)


    @mock.patch.object(DataLcDegree, "get_data")
    @mock.patch.object(DataLcHms, "get_data")
    def test(self,mock1,mock2):
        str_test = 'Ligth Curve Data'
        mock1.return_value = str_test
        mock2.return_value = str_test

        self.assertEqual(self.test_get_data_degree_all.get_data(), str_test)
        self.assertEqual(self.test_get_data_degree_one.get_data(), str_test)
        self.assertEqual(self.test_get_data_hms_all.get_data(), str_test)
        self.assertEqual(self.test_get_data_hms_one.get_data(), str_test)



        
if __name__ == '__main__':
    unittest.main() 




from Freya_alerce.core.commands.base_api import BaseAPI


class NewAPI(BaseAPI):
    """
    Created new FreyaAPI.
    """
    def __init__(self,**kwargs):
        super().__init__(**kwargs)
        super().create_new_api()


from Freya_alerce.catalogs.core.core import GetData

class DataLcDegree(GetData):
    """
    Parameters:
    ------------
    ra : (float) 
        Right ascension | example: ra=139.33444972
    dec :  (float) 
        Declination | example: dec=68.6350604
    radius: (float) 
        Search radius | example: radius=0.0002777
    format: (string) 
        [numpy,csv, votable] | example: format='votable'
    nearest: (bool)
        True or False | example: nearest=True
    """
    def __init__(self,**kwargs):
        super().__init__(**kwargs)
    
    def get_data(self):
        """
        Return
        ------
        Return the LCs from the selected catalog.
        """
        return super().generic_call_data('get_lc_deg')

class DataLcHms(GetData):
    """
    Parameters:
    ------------
    hms : (string) 
        ICRS | example: hms = '9h17m20.26793280000689s +4h34m32.414496000003936s'
    radius: (float) 
        Search radius | example: radius=0.0002777
    format: (string) 
        [numpy,csv, votable] | example: format='csv'
    nearest: (bool)
        True or False | example: nearest=False
    """
    def __init__(self,**kwargs):
        super().__init__(**kwargs)

    def get_data(self):
        """
        Return
        ------
        Return the LCs from the selected catalog.
        """
        return super().generic_call_data('get_lc_hms')


#command Freya
from Freya_alerce.core.commands.addcatalog import AddCatalog
from Freya_alerce.core.commands.addcataloglocal import AddCatalogLocal
from Freya_alerce.core.commands.addresource import AddResource
from Freya_alerce.core.commands.newapi import NewAPI
from Freya_alerce.core.commands.renamecatalog import RenameCatalog
from Freya_alerce.core.commands.deletecatalog import DeleteCatalog
#
import Freya_alerce.catalogs # __path__
#
import os
import argparse
import sys

"""
Define the CLI for Freya specific command line. 
"""
class Main(object):

    def main():

        #----------------------------COMMAND LINE----------------------------------------------#
        #parser = argparse.ArgumentParser()
        formatter = lambda prog: argparse.HelpFormatter(prog,max_help_position=75)
        parser = argparse.ArgumentParser(formatter_class=formatter)
        #--------------------------------------------------------------------------------------#

        #--------------------------------------------------------------------------------------#
        parser.add_argument('-nc','--newcatalog', action='store', 
                            metavar=('<name>','<source>'),
                            help="add new catalog inside Freya",
                            type=str, nargs=2)
        #--------------------------------------------------------------------------------------#
        parser.add_argument('-ncl','--newcataloglocal', action='store', 
                            metavar=('<name>','<source>'),
                            help="add new catalog who local module",
                            type=str, nargs=2)
        #--------------------------------------------------------------------------------------#
        parser.add_argument('-rc','--renamecatalog', action='store', 
                            metavar=('<old_name>','<new_name'),
                            help="rename catalog inside Freya",
                            type=str, nargs=2)
        #--------------------------------------------------------------------------------------#
        parser.add_argument('-dc','--deletecatalog', action='store', 
                            metavar=('<name>'),
                            help="delete catalog inside Freya",
                            type=str, nargs=1)
        #--------------------------------------------------------------------------------------# 
        parser.add_argument('-na','--newapi', action='store_true', 
                            help="create a new FreyaAPI")
        #--------------------------------------------------------------------------------------#                        
        parser.add_argument('-ar','--addresource', action='store',
                            metavar='<name>',
                            help="add module catalog who resource in FreyaApi",
                            type=str, nargs=1)
        #--------------------------------------------------------------------------------------#
        args = parser.parse_args()

        """
        Check what was the command line called, try call associated method.
        """
        if args.newcatalog :
            print("Created new catalogue...")
            try:
                AddCatalog(name=args.newcatalog[0],source=args.newcatalog[1],path=Freya_alerce.catalogs.__path__[0])
            except:
                raise TypeError (f'Failed to create new catalogue : {args.newcatalog[0]} inside Freya')

        elif args.newcataloglocal : 
            print("Created new local catalogue...")
            try:
                AddCatalogLocal(name=args.newcataloglocal[0],source =args.newcataloglocal[1],path=os.getcwd())
            except:
                raise TypeError ('Fauled to create local module')

        elif args.renamecatalog : 
            print("Rename catalogue...")
            try:
                RenameCatalog(name=args.renamecatalog[0],new_name=args.renamecatalog[1],path=Freya_alerce.catalogs.__path__[0])
            except:
                raise TypeError ('Fauled to raname')
        
        elif args.deletecatalog : 
            print("Delete catalogue...")
            try:
                DeleteCatalog(name=args.deletecatalog[0],path=Freya_alerce.catalogs.__path__[0])
            except:
                raise TypeError ('Fauled to delete')

        elif args.newapi :
            print("Created new FreyaAPI...")
            try:
                NewAPI(path=os.getcwd())
            except:
                raise TypeError ('Failed to create new base to FreaAPI')

        elif args.addresource : 
            print("Add new resource to FreyaAPI...")
            try:
                AddResource(name=args.addresource[0],path=os.getcwd())
            except:
                raise TypeError (f'Failed to create resouce : {args.addresource[0]} inside FreyaAPI')


if __name__ == ' __main__':
          Main().main()


import importlib
import io
import numpy as np
import pandas as pd
from astropy.io import ascii
from astropy.io.votable import parse,parse_single_table,from_table, writeto
from astropy.io.votable.tree import VOTableFile, Resource, Table, Field
from astropy.table import Table, Column
from Freya_alerce.files.verify_file import Verify

"""
Class to get data from module catalog configured in Freya, first check if catalog exist inside Freya
and if not exist try import catalog from local folder. The data get is all ligh curve of object in area use
degrees (ra,dec,radius) or use the format hh:mm:ss (hh:mm:ss,radius).
Other option is get the only light curve of object most close to area selected.
"""
class GetData(object):
    """
    Parameters
    -------------------------------------- 
    ra : (float) 
        (degrees) Right Ascension
    dec : (float) 
        (degrees) Declination 
    hms : (string)
        format ICRS (hh:mm:ss)
    radius : (float)
        Search radius
    format : (string)
        [numpy,csv,votable]
    catalog: (string)
        Catalog to search
    """

    def __init__(self,radius=0.0002777,format='numpy',nearest=False,**kwargs):
        self.catalog = kwargs.get('catalog').strip().upper()
        #self.catalog = self.catalog.replace(self.catalog[0],self.catalog[0].upper(),1)
        self.ra = kwargs.get('ra')
        self.dec = kwargs.get('dec')
        self.hms = kwargs.get('hms')
        self.radius = radius
        self.format = format
        self.nearest = nearest

        if self.format not in ['csv','votable']:
             return "inadmissible format in consult data"
    

    def generic_call_data(self,call_method):
        """
        Get the LC of catalog called in format CSV/VOTable. 
        Return
        -------
        Columns : ['obj','ra','dec','mjd','mag','filter','catalog'].
            obj : double
                Id of object in catalog
            ra : float
                Right ascension
            dec : float
                Declination
            mjd : float
                Julian day
            mag : float
                Magnitud
            magerr : float
                Magnitud error
            filter : str
                Band
            catalog : str
                Catalog source
        """
        try :
            """
            Search catalog insiede Freya, if not exist search inside local folder.
            """
            if Verify().verify_catalog_inside(self.catalog):
                module = f'Freya_alerce.catalogs.{self.catalog}.configure'
            elif Verify().verify_catalog_local(self.catalog) :
                 module = f'{self.catalog}.configure'

            # Import self.catalog
            mod = importlib.import_module(module)
            # Call class
            class_ =  getattr(mod,f'Configure{self.catalog}') 
            # Call method especific of class
            if call_method == 'get_lc_deg':
                method_ = class_(ra=self.ra,dec=self.dec,radius=self.radius,nearest=self.nearest).get_lc_deg()      
            elif call_method == 'get_lc_hms':
                method_ = class_(hms=self.hms,radius=self.radius,nearest=self.nearest).get_lc_hms()
            # set de estructure return with format
            if self.format == 'numpy':
                row_catalog = np.full(method_.shape[0],self.catalog)
                method_ = np.column_stack((method_, row_catalog))
                return method_
            elif self.format == 'csv':
                row_catalog = np.full(method_.shape[0],self.catalog)
                df = pd.DataFrame({'obj':method_[:,0],'ra':method_[:,1],
                                    'dec':method_[:,2],'mjd':method_[:,3],
                                    'mag':method_[:,4],'magerr':method_[:,5],
                                    'filter':method_[:,6],'catalog':row_catalog})
                return df.to_csv(index=False)
            elif self.format == 'votable':
                row_catalog = np.full(method_.shape[0],self.catalog)
                method_ = np.column_stack((method_, row_catalog))
                names_column = ['obj','ra','dec','mjd','mag','magerr','filter','catalog']
                descriptions_column = ['Id of object in catalog the original catalog',
                                        'Right ascension','Declination',
                                        'Julian Day','Magnitude','Magnitude Error',
                                        'Filter code','Original Catalog']
                #dtype_column = [] # dtype=dtype_column
                t = Table(rows=method_,names=names_column,descriptions=descriptions_column)
                votable= VOTableFile.from_table(t)
                buf = io.BytesIO()
                writeto(votable,buf)
                return buf.getvalue().decode("utf-8")
        except :
            print(f'No find the catalog : {self.catalog}')



import numpy as np
from astropy.coordinates import SkyCoord
from astropy import units as u
"""
This class represent the generic methods
"""
class Utils:

    def deg_to_hms(self,ra,dec):
        """
        Transform degree point(ra,dec) in IRCS point(hms).
        Parameters
        ----------
        ra : float

        dec: float
        Return
        ---------
        Return point in string
        """
        coord_icrs = SkyCoord(ra=ra, dec=dec)
        return c.to_string('hmsdms')

    def hms_to_deg(self,hms):
        """
        Transform IRCS point(hms) in degree point(ra,dec).
        Parameters
        ----------
        hms : string
        Return
        ---------
        Return two float values: 'ra' and 'deg'
        """
        coord = SkyCoord(hms,frame='icrs') #transform coord
        ra = coord.ra.degree
        dec = coord.dec.degree
        return ra,dec

    def get_nearest(self,center_ra,center_dec,matrix_ra_dec):
        """
        Transform degree point(ra,dec) in IRCS point(hms).
        Parameters
        ----------
        center_ra : float
        
        center_dec: float

        matrix_ra_dec : numpy array 2d values
            matrix in two dimension with column ra dec and the row represent the object, 
        Return
        ----------
        Return the index of matrix for the min angle between ra | dec center point and ra | dec the object.
        """
        angle = []
        c1 = SkyCoord(ra=center_ra,dec=center_dec,unit=u.degree)
        for obj in matrix_ra_dec:
            c2 = SkyCoord(obj[0],obj[1],unit=u.degree)
            angle.append(c1.separation(c2))
        return angle.index(min(angle))

    def flux_to_mag(self,flux):
        """
        Convert flux in Jy to magnitudes, use -2.5*np.log10(flux) + 8.90 formule.
        Parameters
        ----------
        flux : numpy array
            flux in Jy
        
        Return
        ----------
        Return numpy array with magnitude 
        """
        mag = -2.5*np.log10(flux) + 8.90
        return mag



from unittest import TestCase

class TestImport(TestCase):
    
    def test(self):
        try:
            from Freya_alerce.core.commands.base_freya import Base
            from Freya_alerce.core.commands.base_api import BaseAPI
            from Freya_alerce.files.list_file import ListFiles
            from Freya_alerce.files.verify_file import Verify
            from Freya_alerce.catalogs.core.data_lc import DataLcDegree,DataLcHms
        except OSError as error:
            print(error)  

if __name__ == '__main__':
    unittest.main() 


"""
configure.py is the most important file in Freya, this file is called for 
Freya’s core and FreyaAPI’s resources, is the only file you need to modify 
in principle. You need to complete the following methods. When using Freya’s 
method getData, you use this class for calls and depent what method call you 
need use ra and dec or hms, so that's why kwargs is used.
"""
import requests
import io
import numpy as np
from astropy.io import ascii

from Freya_alerce.catalogs.core.abstract_catalog import BaseCatalog
from Freya_alerce.core.utils import Utils

class ConfigureZTF(BaseCatalog):
    """
    Parameters:
    ------------
    ra : (float) 
        Right ascension
    dec :  (float) 
        Declination
    hms : (string) 
        ICRS
    radius: (float) 
        Search radius
    nearest: (bool)
        True or False
    """
    def __init__(self,**kwagrs):
        self.ra = kwagrs.get('ra')
        self.dec = kwagrs.get('dec')
        self.hms = kwagrs.get('hms')
        self.radius = kwagrs.get('radius')
        self.nearest = kwagrs.get('nearest')

    def id_nearest (self,results):
        """ Get the idex of object id most closet to ra dec use a min angle
        """
        matrix = []
        for group in results.groups:
            matrix.append([group['ra'][0],group['dec'][0]])
        matrix = np.array(matrix)
        return_ = Utils().get_nearest(self.ra,self.dec,matrix)
        return return_ 

    def get_matrix_data(self,result):
        ztfdic = ''
        result_ = ascii.read(result.text)
        if len(result_) <= 0:
            ztfdic = 'light curve not found' 
            return ztfdic

        #the most close object to radius
        if self.nearest is True:
            
            result_ = result_.group_by('oid')
            minztf = self.id_nearest(result_)
            ztf_matrix = np.array([result_.groups[minztf]['oid'],result_.groups[minztf]['ra'],
                                    result_.groups[minztf]['dec'],result_.groups[minztf]['mjd'],
                                    result_.groups[minztf]['mag'],result_.groups[minztf]['magerr'],
                                    result_.groups[minztf]['filtercode']]).T
            return ztf_matrix
        # all objects in radius
        else:
            ztf_matrix = np.array([result_['oid'],result_['ra'],result_['dec'],result_['mjd'],
                                    result_['mag'],result_['magerr'],result_['filtercode']]).T
            return ztf_matrix

    def zftcurves(self):
        """ Get light curves of ztf objects 
        Parameters
        ----------
        """
        baseurl="https://irsa.ipac.caltech.edu/cgi-bin/ZTF/nph_light_curves"
        data = {}
        data['POS']=f'CIRCLE {self.ra} {self.dec} {self.radius}'
        #data['BANDNAME']='r'
        data['FORMAT'] = 'csv'
        result = requests.get(baseurl,params=data)
        ztfdic = ''
        #return result
        if result.status_code != 200: 
            ztfdic = result.status_code 
            return ztfdic
        else:
            return self.get_matrix_data(result)

    def get_lc_deg(self):
        """
        Get all ligth curves data or the most close object,inside degree area from ZTF catalog.
        Return
        -------
        Return numpy array 2d with rows represent the objects and columns : ['obj','ra','dec','mjd','mag','magerr,'filter'].
            obj : double
                Id of object in catalog
            ra : float
                Right ascension
            dec : float
                Declination
            mjd : float
                Julian day
            mag : float
                Magnitud
            magerr : float
                Magnitud error
            filter : str
                Band
        """
        data_return = self.zftcurves() 
        return data_return
    
    def get_lc_hms(self):
        """
        Get all ligth curves data or the most close object, inside hh:mm:ss area from ZTF catalog
        Return
        -------
        Return numpy array 2d with rows represent the objects and columns : ['obj','ra','dec','mjd','mag',magerr,'filter'].
            obj : double
                Id of object in catalog
            ra : float
                Right ascension
            dec : float
                Declination
            mjd : float
                Julian day
            mag : float
                Magnitud
            magerr : float
                Magnitud error
            filter : str
                Band
        """
        ra_,dec_ = Utils().hms_to_deg(self.hms)
        self.ra = ra_
        self.dec = dec_
        data_return = self.zftcurves() 
        return data_return



import Freya_alerce.catalogs # __path__ # dir modules
import os
import importlib

class Verify(object):

    def verify_catalog_inside(self,name):
        """
        Verify name catalog already exist inside Freya.
        Parameters
        ----------
        name : string
            Name to search.
        Return: bool
        ----------
        Return True if catalog exists inside Freya in other case
        return False.
        """
        self.name = name
        dir_catalogs = Freya_alerce.catalogs.__path__[0]
        if self.name  in os.listdir(dir_catalogs) :
             return True 
        return False

    def verify_catalog_local(self,name):
        """
        Verify if catalogue path exist who a package in PYTHONPATH.
        Parameters
        ----------
        name : string
            Name to search.
        Return: bool
        ----------
        Return True if catalog exists in other case
        return False.
        """
        self.name = name
        try:
            mod = importlib.import_module(f'{self.name}')
            return True
        except:
            return False
    
    def verify_source(self,source):
        """
        Verify if source of catalog is permitted who source.
        Parameters
        ----------
        source : string
            valid source [api,db,other]
        Return: bool
        ----------
        Return True if source not is valid.
        """
        self.source = source
        if self.source not in ['api','db','other']:
            return True
        return False



from Freya_alerce.core.commands.base_freya import Base


class AddCatalog(Base):
    """
    Add new catalogue inside Freya.

    Parameters
    ----------
    name : (string) 
        name with add catalogue inside Freya.
    source : (string) 
        origin source catalogue [api,db].
    """
    def __init__(self,**kwargs):
        super().__init__(**kwargs)
        super().create_module_catalog()


from setuptools import setup,find_packages

# pip install -e .
setup(
      name='Freya_alerce',
      version='0.1.4.8.8',
      author='Jonimott de Malpais',
      author_email='',
      description='Freya is a Python framework that quick development queries in astronomical catalogs and use local or creating easy new API called FreyaAPI',
      url='',
      license='',
      packages=find_packages(),
      #include_package_data=True,
      package_data = {
            'Freya_alerce.core': ['*.txt'],
            'Freya_alerce.files.file_templates': ['*.zip']
      },
      entry_points = {
            'console_scripts': ['freya-admin=Freya_alerce.freya:Main.main'],
      },
      zip_safe=False,
      classifiers=[
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Natural Language :: English"

      ],
      python_requires='>=3.9',
      install_requires=['astropy==4.2',
                        'pandas==1.2.0',
                        'requests==2.25.1',
                        'GitPython==3.1.13']
)


import tempfile

from unittest import TestCase
from Freya_alerce.core.commands.base_freya import Base
from Freya_alerce.files.list_file import ListFiles

class TestRenameCatalog(TestCase):

    def setUp(self):
        self.tmp_test = tempfile.TemporaryDirectory()

    def test(self):
        new_catalog = Base(name='test',source='api',path=self.tmp_test.name).create_module_catalog()
        Base(name='test',new_name='test2',path=self.tmp_test.name).rename_catalog()

        new_catalog = Base(name='test3',source='db',path=self.tmp_test.name).create_module_catalog()
        Base(name='test3',new_name='test4',path=self.tmp_test.name).rename_catalog()
    def tearDown(self):
        self.tmp_test.cleanup()

if __name__ == '__main__':
    unittest.main() 


from Freya_alerce.core.commands.base_freya import Base

class DeleteCatalog(Base):
    """
    Delete catalog inside Freya

    Parameters
    ----------
    name : (string) 
        name of catalog to delete inside Freya.
    """
    def __init__(self,**kwargs):
        super().__init__(**kwargs)
        super().delete_catalog()